import pytz
from datetime import datetime, timedelta

_korean_tz = pytz.timezone('Asia/Seoul')

def datetime_sub(days=0):
    return datetime.now(_korean_tz) - timedelta(days=days)

def format_yesterday(format='%Y%m%d'):
    return datetime_sub(1).strftime(format)

def get_validate_query(df, database, table, dialect='mysql'):
    sub = df[df.TABLE_NAME == table]
    parts = []
    for _, row in sub.iterrows():
        col, typ = row.COLUMN_NAME, row.COLUMN_TYPE
        if typ.startswith(('bigint','int','smallint','tinyint','mediumint')):
            parts.append(f"SUM({col}) AS {col}")
        elif typ.startswith('bit'):
            if dialect == 'mysql':
                parts.append(f"SUM(CAST({col} AS UNSIGNED)) AS {col}")
            else:
                parts.append(f"SUM({col}) AS {col}")
        elif typ.startswith(('varchar','longtext','text')):
            if dialect == 'mysql':
                parts.append(f"SUM(LENGTH({col})) AS {col}")
            else:
                parts.append(f"SUM(BYTE_LENGTH({col})) AS {col}")
        elif typ == 'json':
            if dialect == 'mysql':
                parts.append(f"SUM(LENGTH(REPLACE(CAST({col} AS CHAR),' ',''))) AS {col}")
            else:
                parts.append(f'''
                SUM(BYTE_LENGTH(
                if({col} is null, null, replace(TO_JSON_STRING({col}),' ','')))
                ) AS {col}''')
        elif typ.startswith(('datetime')):
            if dialect == 'mysql':
                parts.append(f"SUM(cast(UNIX_TIMESTAMP({col}) * 1000000 as UNSIGNED)) AS {col}")
            else:
                parts.append(f"SUM(UNIX_micros(TIMESTAMP({col}))) AS {col}")
        elif typ.startswith(('timestamp')):
            if dialect == 'mysql':
                parts.append(f"SUM(UNIX_TIMESTAMP({col})) AS {col}")
            else:
                parts.append(f"SUM(UNIX_micros({col})) AS {col}")
        else:
            parts.append(f"COUNT(1) AS {col}")
    select_clause = ", ".join(parts)
    return f"SELECT count(1) cnt, {select_clause} FROM {database}.{table}"